/*
**  Copyright (C) 1993-1996, OFFIS, Oldenburg University and CERIUM
**
**  This software and supporting documentation were
**  developed by
**
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**
**  for CEN/TC251/WG4 as a contribution to the Radiological
**  Society of North America (RSNA) 1993 Digital Imaging and
**  Communications in Medicine (DICOM) Demonstration.
**
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE
**  IS WITH THE USER.
**
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice.
**
*/

/*
 *
 * Author: Jens Burkert	/ Andrew Hewett		Created: 11-08-93
 * 	   Oldenburg University
 *	   Germany
 *
 * Module: dispuser
 *
 * Purpose:
 * This file contains the routines which provide
 * user side access to the CTN display.  The implementation
 * translates these function calls into messages to send to
 * the CTN Display Process.
 *
 * These routines are intended to be called from the CTN or from
 * the TI so that visitors to the RSNA'93 DICOM Demonstration
 * can see what activities are taking place.
 *
 * Module Prefix: DSU_
 *
 * Last Update:		$Author: joergr $
 * Revision:		$Revision: 1.15 $
 * Status:		    $State: Exp $
 *
 * $Id: dispuser.cc,v 1.15 2001/06/05 10:10:18 joergr Exp $
 *
 */

#include "osconfig.h"

#include <stdio.h>
#include <string.h>

#ifdef HAVE_STDLIB_H
#ifndef  _BCB4
/* workaround for bug in Borland C++ Builder 4 */
BEGIN_EXTERN_C
#endif
#include <stdlib.h>
#ifndef  _BCB4
END_EXTERN_C
#endif
#endif

extern "C"
{

#ifdef HAVE_LIBC_H
#include <libc.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_USLEEP
#ifndef HAVE_PROTOTYPE_USLEEP
     int usleep(unsigned int useconds);
#endif
#endif
}

#include "snq.h"
#include "dispuser.h"
#include "disppriv.h" /* defines not of public interest */
#include "dcompat.h"  /* for sleep() and getpid() on Win32 */
#include "ofconsol.h"

typedef struct
{
    int active;
    char peer[MAX_STRING_LENGTH];
    char peerAE[MAX_STRING_LENGTH];
    char myAE[MAX_STRING_LENGTH];
    DSU_AssociationType aType;
} DSU_Association;

static int dsu_ignore = 0;
#ifdef DEBUG
 static DSU_UserType dsu_utype = DSU_CTNUser;	/* default */
#endif
static DSU_Association dsu_assocs[3];
static QUEUE_Element dsu_CTNdisplayhandle[3];
/* This is the default queue number. You may change it in your program,
** but you must init ctndisp with that changed Queue number.
** (ctndisp -q queue_number)
*/
static int QUEUENUMBER=0;

#ifdef DEBUG
static int traceLevel = 0;
#endif

#ifdef DEBUG
/* used for diagnostic output only */
static const char *dsu_userString[] = { "CTN", "TI" };
static const char *dsu_levelString[] = { "Main", "Sub", "None" };
static const char *dsu_opString[] = { "Store", "Find", "Move", "Print" };
static const char *dsu_stateString[] = { "Start", "Running", "Stop" };
static const char *dsu_directionString[] = { "In", "Out" };
#endif


void SendToCtndisp(DSU_AssociationLevel aLevel)
{
  SNQ_code cond;

  cond = SNQ_GetQueue(dsu_CTNdisplayhandle[aLevel].Q_QueueID);
  while( ((cond = SNQ_Enqueue(&dsu_CTNdisplayhandle[aLevel])) != SNQ_NORMAL) &&
          (cond !=SNQ_FILEACCESSFAIL) && (cond != SNQ_NOMEMORY) && (cond != SNQ_UNIMPLEMENTED))
  {
#ifdef HAVE_USLEEP
    usleep(100000);	/* sleep 0.1 sec */
#else
    sleep(1);
#endif
  }
  if (cond != SNQ_NORMAL)
  {
    if(cond == SNQ_QUEUEFULL) CERR << "Error: SNQ_QUEUEFULL" << endl;
    if(cond == SNQ_QUEUEEMPTY) CERR << "Error: SNQ_QUEUEEMPTY" << endl;
    if(cond == SNQ_SHAREDMEMORYFAIL) CERR << "Error: SNQ_SHAREDMEMORYFAIL" << endl;
    if(cond == SNQ_SEMAPHOREFAIL) CERR << "Error: SNQ_SEMAPHOREFAIL" << endl;
    if(cond == SNQ_FILEACCESSFAIL) CERR << "Error: SNQ_FILEACCESSFAIL" << endl;
    if(cond == SNQ_NOMEMORY) CERR << "Error: SNQ_NOMEMORY" << endl;
    if(cond == SNQ_UNIMPLEMENTED) CERR << "Error: SNQ_UNIMPLEMENTED" << endl;
    CERR << "Unable to Enqueue Operation : " << dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation
        << " in SNQ_Enqueue, CONDITION : " << (int) cond << endl;
  } else {
#ifdef DEBUG
    if (traceLevel > 0) {
      CERR << "Enqueue Image successful AssociationID: "
        << dsu_CTNdisplayhandle[aLevel].Q_AssociationID << endl;
    }
#endif
  }
  return;
}

#ifdef DEBUG
void DSU_setTraceLevel(int aTraceLevel)
{
  traceLevel = aTraceLevel;
}
#else
void DSU_setTraceLevel(int /*aTraceLevel*/)
{
}
#endif


int DSU_initDisplay(DSU_UserType userType)
{
  SNQ_code cond;
#ifdef DEBUG
  dsu_utype = userType;
#endif
  int success=0;

  dsu_assocs[DSU_MainAssociation].active = 0;
  dsu_assocs[DSU_SubAssociation].active = 0;
  dsu_assocs[DSU_NoAssociation].active = 0;

  if (dsu_ignore) return 1;

  cond = SNQ_GetQueue(QUEUENUMBER);
  dsu_CTNdisplayhandle[0].Q_QueueID = QUEUENUMBER;
  dsu_CTNdisplayhandle[1].Q_QueueID = QUEUENUMBER;
  dsu_CTNdisplayhandle[2].Q_QueueID = QUEUENUMBER;
  if (cond == SNQ_NORMAL) {
    if (userType == DSU_CTNUser) {
      dsu_CTNdisplayhandle[0].Q_DSU_UserType = CTNUser;
      dsu_CTNdisplayhandle[1].Q_DSU_UserType = CTNUser;
      dsu_CTNdisplayhandle[2].Q_DSU_UserType = CTNUser;
    } else {
       dsu_CTNdisplayhandle[0].Q_DSU_UserType = TIUser;
       dsu_CTNdisplayhandle[1].Q_DSU_UserType = TIUser;
       dsu_CTNdisplayhandle[2].Q_DSU_UserType = TIUser;
    }
#ifdef DEBUG
    if (traceLevel > 0) {
      CERR << "CTNDISPLAY detected" << endl;
    }
#endif
    success=1;
  } else {
    CERR << "WARNING: CTNDISPLAY NOT detected !" << endl
        << "Maybe wrong queue number or ctndisp not running." << endl
        << "Communication with CTNDISPLAY disabled." << endl;
    dsu_ignore = 1;
  }
  dsu_CTNdisplayhandle[DSU_MainAssociation].Q_DSU_AssociationType = 0;
  dsu_CTNdisplayhandle[DSU_MainAssociation].Q_DSU_AssociationLevel = 0;
  dsu_CTNdisplayhandle[DSU_MainAssociation].Q_DSU_Operation = InitOperation;
  dsu_CTNdisplayhandle[DSU_MainAssociation].Q_DSU_OperationState = 0;
  dsu_CTNdisplayhandle[DSU_MainAssociation].Q_DSU_Direction = 0;
  dsu_CTNdisplayhandle[DSU_MainAssociation].Q_DSU_Percent = 0;
  dsu_CTNdisplayhandle[DSU_MainAssociation].Q_active = 0;
  strcpy(dsu_CTNdisplayhandle[DSU_MainAssociation].Q_peer, "");
  strcpy(dsu_CTNdisplayhandle[DSU_MainAssociation].Q_peerAE, "");
  strcpy(dsu_CTNdisplayhandle[DSU_MainAssociation].Q_myAE, "");
  strcpy(dsu_CTNdisplayhandle[DSU_MainAssociation].Q_ImageName, "");
  dsu_CTNdisplayhandle[DSU_SubAssociation].Q_DSU_AssociationType = 0;
  dsu_CTNdisplayhandle[DSU_SubAssociation].Q_DSU_AssociationLevel = 0;
  dsu_CTNdisplayhandle[DSU_SubAssociation].Q_DSU_Operation = InitOperation;
  dsu_CTNdisplayhandle[DSU_SubAssociation].Q_DSU_OperationState = 0;
  dsu_CTNdisplayhandle[DSU_SubAssociation].Q_DSU_Direction = 0;
  dsu_CTNdisplayhandle[DSU_SubAssociation].Q_DSU_Percent = 0;
  dsu_CTNdisplayhandle[DSU_SubAssociation].Q_active = 0;
  strcpy(dsu_CTNdisplayhandle[DSU_SubAssociation].Q_peer, "");
  strcpy(dsu_CTNdisplayhandle[DSU_SubAssociation].Q_peerAE, "");
  strcpy(dsu_CTNdisplayhandle[DSU_SubAssociation].Q_myAE, "");
  strcpy(dsu_CTNdisplayhandle[DSU_SubAssociation].Q_ImageName, "");
  dsu_CTNdisplayhandle[DSU_NoAssociation].Q_DSU_AssociationType = 0;
  dsu_CTNdisplayhandle[DSU_NoAssociation].Q_DSU_AssociationLevel = 0;
  dsu_CTNdisplayhandle[DSU_NoAssociation].Q_DSU_Operation = InitOperation;
  dsu_CTNdisplayhandle[DSU_NoAssociation].Q_DSU_OperationState = 0;
  dsu_CTNdisplayhandle[DSU_NoAssociation].Q_DSU_Direction = 0;
  dsu_CTNdisplayhandle[DSU_NoAssociation].Q_DSU_Percent = 0;
  dsu_CTNdisplayhandle[DSU_NoAssociation].Q_active = 0;
  strcpy(dsu_CTNdisplayhandle[DSU_NoAssociation].Q_peer, "");
  strcpy(dsu_CTNdisplayhandle[DSU_NoAssociation].Q_peerAE, "");
  strcpy(dsu_CTNdisplayhandle[DSU_NoAssociation].Q_myAE, "");
  strcpy(dsu_CTNdisplayhandle[DSU_NoAssociation].Q_ImageName, "");
  return success;
}

void DSU_setDisplayMode(int sendMessages)
{
  dsu_ignore = !sendMessages;
}

void DSU_startAssociation(
  const char *peerHostName,
  const char *peerAETitle,
  const char *myAETitle,
  DSU_AssociationType aType,
  DSU_AssociationLevel aLevel)
{

#ifdef DEBUG
  if (traceLevel > 0) {
    CERR << "DSU_startAssociation ("
        << dsu_userString[dsu_utype] << "." << dsu_levelString[aLevel] << "): ";
    switch (aType)
    {
    case DSU_Requestor:
	CERR << myAETitle << " -> " << peerHostName << ":" << peerAETitle << endl;
	break;
    case DSU_Acceptor:
	CERR << myAETitle << " <- " << peerHostName << ":" << peerAETitle << endl;
	break;
    }
  }
#endif

  dsu_assocs[aLevel].active = 1;
  strcpy(dsu_assocs[aLevel].peer, peerHostName);
  strcpy(dsu_assocs[aLevel].peerAE, peerAETitle);
  strcpy(dsu_assocs[aLevel].myAE, myAETitle);
  dsu_assocs[aLevel].aType = aType;

  /* real work starts here */
  if (dsu_ignore) return;

  dsu_CTNdisplayhandle[DSU_MainAssociation].Q_AssociationID = (int)getpid();
  dsu_CTNdisplayhandle[DSU_SubAssociation].Q_AssociationID = (int)getpid(); /* Number of the process */
  dsu_CTNdisplayhandle[DSU_NoAssociation].Q_AssociationID = (int)getpid();	 /* Number of the process for Association ident. */

  switch (aType)
  {
    case DSU_Requestor:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_AssociationType = Requestor;
      break;
    case DSU_Acceptor:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_AssociationType = Acceptor;
      break;
  }
  switch (aLevel)
  {
    case DSU_MainAssociation:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_AssociationLevel = MainAssociation;
      break;
    case DSU_SubAssociation:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_AssociationLevel = SubAssociation;
      break;
    case DSU_NoAssociation:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_AssociationLevel = NoAssociation;
      break;
  }
  dsu_CTNdisplayhandle[aLevel].Q_active = 1;
  dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation=StartOperation;
  strcpy(dsu_CTNdisplayhandle[aLevel].Q_peer, peerHostName);
  strcpy(dsu_CTNdisplayhandle[aLevel].Q_peerAE, peerAETitle);
  strcpy(dsu_CTNdisplayhandle[aLevel].Q_myAE, myAETitle);
  SendToCtndisp(aLevel); /* Send the Handle */
}


void DSU_stopAssociation(DSU_AssociationLevel aLevel)
{
#ifdef DEBUG
  if (traceLevel > 0) {
    CERR << "DSU_stopAssociation ("
        << dsu_userString[dsu_utype] << "." << dsu_levelString[aLevel] << ")" <<endl;
  }
#endif
  dsu_assocs[aLevel].active = 0;
  if (dsu_ignore) return;

  dsu_CTNdisplayhandle[aLevel].Q_active = 0;
  dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation=StopOperation;
  SendToCtndisp(aLevel); /* Send the Handle */
  return;
}

void DSU_operationState(DSU_AssociationLevel aLevel,
	DSU_Operation op, DSU_OperationState state,
	DSU_Direction direction, int percentComplete)
{
#ifdef DEBUG
  if (traceLevel > 0) {
    CERR << "DSU_operationState ("
        << dsu_userString[dsu_utype] << "." << dsu_levelString[aLevel] << "): "
        << dsu_opString[op] << " " << dsu_directionString[direction] << " " << dsu_stateString[state]
        << " " << percentComplete << "%" << endl;
  }
#endif

  /* real work starts here */
  if (dsu_ignore) return;
  switch (op)
  {
    case DSU_StoreOperation:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation =StoreOperation ;
      break;
    case DSU_FindOperation:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation = FindOperation;
      break;
    case DSU_MoveOperation:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation = MoveOperation;
      break;
    case DSU_GetOperation:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation = GetOperation;
      break;
    case DSU_PrintOperation:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation = PrintOperation;
      break;
    case DSU_KillOperation:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation = KillOperation;
      break;
  }

  if (dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation==KillOperation)
    dsu_CTNdisplayhandle[aLevel].Q_DSU_OperationState = KillOperation;
  else switch (state)
  {
    case DSU_StartOperation:
	dsu_CTNdisplayhandle[aLevel].Q_DSU_OperationState = StartOperation;
	break;
    case DSU_RunningOperation:
	dsu_CTNdisplayhandle[aLevel].Q_DSU_OperationState = RunningOperation;
	break;
    case DSU_StopOperation:
	dsu_CTNdisplayhandle[aLevel].Q_DSU_OperationState = StopOperation;
	break;
  }

  switch (direction)
  {
    case DSU_Incoming:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_Direction =Incoming ;
      break;
    case DSU_Outgoing:
      dsu_CTNdisplayhandle[aLevel].Q_DSU_Direction = Outgoing;
      break;
  }
  dsu_CTNdisplayhandle[aLevel].Q_DSU_Percent=percentComplete;
  SendToCtndisp(aLevel); /* Send the Handle */
  return;
}


void
DSU_displayImage(DSU_AssociationLevel aLevel,
		 const char *imagePathName)
{
#ifdef DEBUG
  if (traceLevel > 0) {
    CERR << "DSU_displayImage ("
        << dsu_userString[dsu_utype] << "." << dsu_levelString[aLevel] << "): "
        << imagePathName << endl;
  }
#endif
  if (dsu_ignore) return;
  if (aLevel == DSU_NoAssociation)
  {
    /* display a Image while there is no association */
    /*DSU_initDisplay(DSU_CTNUser);*/
    DSU_startAssociation(" ", " ", " ", DSU_Requestor, DSU_NoAssociation);
    dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation = ImageOperation;
    strcpy(dsu_CTNdisplayhandle[aLevel].Q_ImageName, imagePathName);
    SendToCtndisp(aLevel);	/* Send the Handle */
    DSU_stopAssociation(DSU_NoAssociation);
  } else {
    dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation = ImageOperation;
    strcpy(dsu_CTNdisplayhandle[aLevel].Q_ImageName, imagePathName);
    SendToCtndisp(aLevel);	/* Send the Handle */
  }
  return;
}

void
DSU_displayIndependent(
  const char *ImageTitle,
  const char *imagePathName)
{
  DSU_AssociationLevel aLevel = DSU_NoAssociation;
#ifdef DEBUG
  if (traceLevel > 0) {
    CERR << "DSU_displayImage ("
        << dsu_userString[dsu_utype] << "." << dsu_levelString[aLevel] << "): "
        << imagePathName << endl;
  }
#endif
  if (dsu_ignore) return;
  /* display a Image while there is no association */
  aLevel = DSU_NoAssociation;
  /*DSU_initDisplay(DSU_CTNUser);*/
  DSU_startAssociation(ImageTitle, " ", " ", DSU_Requestor, DSU_NoAssociation);
  dsu_CTNdisplayhandle[aLevel].Q_DSU_Operation = ImageOperation;
  strcpy(dsu_CTNdisplayhandle[aLevel].Q_ImageName, imagePathName);
  SendToCtndisp(aLevel);	/* Send the Handle */
  DSU_stopAssociation(DSU_NoAssociation);
  return;
}


/*
 * CVS Log
 * $Log: dispuser.cc,v $
 * Revision 1.15  2001/06/05 10:10:18  joergr
 * Minor code purifications to keep Sun CC 2.0.1 quiet.
 *
 * Revision 1.14  2000/03/07 16:21:33  joergr
 * Minor code purifications.
 *
 * Revision 1.13  2000/03/07 16:03:44  joergr
 * Removed unused variable (reported by Sun CC 2.0.1).
 *
 * Revision 1.12  2000/03/06 15:53:06  meichel
 * Introduced typecasts when printing enums to cout/cerr.
 *
 * Revision 1.11  2000/03/03 14:17:57  meichel
 * Implemented library support for redirecting error messages into memory
 *   instead of printing them to stdout/stderr for GUI applications.
 *
 * Revision 1.10  2000/02/23 15:11:23  meichel
 * Corrected macro for Borland C++ Builder 4 workaround.
 *
 * Revision 1.9  2000/02/01 10:49:11  meichel
 * Avoiding to include <stdlib.h> as extern "C" on Borland C++ Builder 4,
 *   workaround for bug in compiler header files.
 *
 * Revision 1.8  1999/06/10 10:55:48  meichel
 * Adapted imagectn to new command line option scheme.
 *   Added support for Patient/Study Only Q/R model and C-GET (experimental).
 *
 * Revision 1.7  1999/05/03 14:18:01  joergr
 * Minor code purifications to keep Sun CC 2.0.1 quiet.
 *
 * Revision 1.6  1999/04/21 15:43:42  meichel
 * Corrected a few typos in ctndisp
 *
 * Revision 1.5  1997/07/24 13:10:48  andreas
 * - Removed Warnings from SUN CC 2.0.1
 *
 * Revision 1.4  1996/05/30 17:23:26  hewett
 * If the display cannot be contacted then all communication requests
 * are subsequently ignored (sets dsu_ignore = 1).
 *
 * Revision 1.3  1996/04/25 16:24:40  hewett
 * Trace information now only printed when traceLevel>0.
 *
 * Revision 1.2  1996/04/18 15:29:50  meichel
 * Version for CAR'96
 *
 * Revision 1.1  1996/03/28 17:08:03  meichel
 * first commit with new file structure
 *
 * Revision 2.1  1995/02/02 12:50:06  hewett
 * Corrected spelling of Ossietzky
 *
 * Revision 2.0  1994/11/17  09:45:17  meichel
 * Version 2.0 for RSNA 1994 demonstration.
 * New features: Metaheader support, new SOP classes, lots of bugs fixed.
 *
 * Revision 1.15  1994/03/17  18:15:31  hewett
 * Changes due to must recent DICOM data dictionary (dcmcode)
 *
 * Revision 1.14  1994/03/17  09:28:19  hewett
 * Updated copyright notice.
 *
 * Revision 1.13  1993/11/22  12:40:54  hewett
 * Now sleeps for 0.1 sec when queue is full (was 1 second)
 *
 * Revision 1.12  1993/11/16  12:25:22  jens
 * Added code for the killassoc command.
 *
 * Revision 1.11  1993/11/10  15:19:01  jens
 * Removed a bug. Now initializing DSU_displayIndependent correct.
 * Removed DSU_initDisplay  in DSU_displayImage and DSU_displayIndependent.
 *
 * Revision 1.10  1993/11/08  12:10:42  hewett
 * Remove comment which SUN acc compiler thought was a tri-graph.
 *
 * Revision 1.9  1993/10/26  16:09:19  jens
 * Added DSU_displayIndependent for sending w/o association
 *
 * Revision 1.8  1993/10/14  10:50:37  hewett
 * Added Euro Header
 *
 * Revision 1.7  1993/09/21  15:44:05  jens
 * You can now display images without association init with
 * DSU_displayImage(DSU_NoAssociation,filename);
 *
 * Revision 1.6  1993/09/07  13:38:59  jens
 * Version with better QUEUE handling.
 *
 * Revision 1.5  1993/09/06  16:38:13  jens
 * new realese with tiny Queue buffering
 *
 * Revision 1.4  1993/08/31  18:33:21  jens
 * Added interface to SNQ routines.
 *
 * Revision 1.3  1993/08/30  09:09:03  hewett
 * Fixed printf bug (was %s instead of %d).
 *
 * Revision 1.2  1993/08/11  14:33:27  hewett
 * Added direction flag
 *
 * Revision 1.1  1993/08/11  13:58:14  hewett
 * Initial Revision
 *
 */
